@@ -1,6 +1,6 @@
 #! /bin/sh
 #
-# This is kernel build script for ubuntu 7.10's 2.6.22 kernel.
+# This is kernel build script for ubuntu 8.04's 2.6.24 kernel.
 #
 
 die () {
@@ -9,6 +9,7 @@ die () {
 }
 
 VERSION=`uname -r | cut -d - -f 1,2`
+export CONCURRENCY_LEVEL=`grep -c '^processor' /proc/cpuinfo` || die "Can't export."
 
 apt-get -y install wget
 for key in 0A0AC927 17063E6D 174BF01A 191FCD8A 60E80B5B 63549F8E 76682A37 8BF9EFE6
@@ -19,9 +20,9 @@ done
 # Download TOMOYO Linux patches.
 mkdir -p /usr/src/rpm/SOURCES/
 cd /usr/src/rpm/SOURCES/ || die "Can't chdir to /usr/src/rpm/SOURCES/ ."
-if [ ! -r ccs-patch-1.5.5-20080903.tar.gz ]
+if [ ! -r ccs-patch-1.5.5-20081111.tar.gz ]
 then
-    wget http://osdn.dl.sourceforge.jp/tomoyo/27219/ccs-patch-1.5.5-20080903.tar.gz || die "Can't download patch."
+    wget http://osdn.dl.sourceforge.jp/tomoyo/27219/ccs-patch-1.5.5-20081111.tar.gz || die "Can't download patch."
 fi
 
 # Install kernel source packages.
@@ -36,27 +37,32 @@ apt-get build-dep linux-restricted-modules-${VERSION}-generic || die "Can't inst
 apt-get source linux-restricted-modules-${VERSION}-generic || die "Can't install kernel source."
 
 # Copy patches and create kernel config.
-cd linux-source-2.6.22-2.6.22/ || die "Can't chdir to linux-2.6.22-2.6.22/ ."
-tar -zxf /usr/src/rpm/SOURCES/ccs-patch-1.5.5-20080903.tar.gz || die "Can't extract patch."
+cd linux-2.6.24/ || die "Can't chdir to linux-2.6.24/ ."
 mkdir -p debian/binary-custom.d/ccs/patchset || die "Can't create debian/binary-custom.d/ccs/patchset ."
-cp -p patches/ccs-patch-2.6.22-ubuntu-7.10.diff debian/binary-custom.d/ccs/patchset/ubuntu-7.10.patch || die "Can't copy patch."
+mkdir -p ccs-patch/ || die "Can't create directory."
+cd ccs-patch/ || die "Can't chdir to ccs-patch/ ."
+tar -zxf /usr/src/rpm/SOURCES/ccs-patch-1.5.5-20081111.tar.gz || die "Can't extract patch."
+cp -p patches/ccs-patch-2.6.24-ubuntu-8.04.diff ../debian/binary-custom.d/ccs/patchset/ubuntu-8.04.patch || die "Can't copy patch."
+rm -fR specs/ patches/ || die "Can't delete directory."
+for i in `find . -type f`; do diff -u /dev/null $i; done > ../debian/binary-custom.d/ccs/patchset/ccs.patch
+cd ../ || die "Can't chdir to ../ ."
 cd debian/binary-custom.d/ccs/ || die "Can't chdir to debian/binary-custom.d/ccs/ ."
-cat ../../config/i386/config ../../config/i386/config.generic ../../../config.ccs > config.i386 || die "Can't create config."
+cat ../../config/i386/config ../../config/i386/config.generic ../../../ccs-patch/config.ccs > config.i386 || die "Can't create config."
 sed -i -e 's:CONFIG_DEBUG_INFO=.*:# CONFIG_DEBUG_INFO is not set:' -- config.i386 || die "Can't edit config."
 touch rules || die "Can't create file."
 cd ../../../ || die "Can't chdir to ../../../ ."
+rm -fR ccs-patch || die "Can't delete directory."
 awk ' BEGIN { flag = 0; print ""; } { if ($1 == "Package:" ) { if (index($0, "-generic") > 0) flag = 1; else flag = 0; }; if (flag) print $0; } ' debian/control.stub | sed -e 's:-generic:-ccs:' > debian/control.stub.ccs || die "Can't create file."
 cat debian/control.stub.ccs >> debian/control.stub || die "Can't edit file."
 debian/rules debian/control || die "Can't run control."
 
 # Start compilation.
-export CONCURRENCY_LEVEL=`grep -c '^processor' /proc/cpuinfo` || die "Can't export."
 debian/rules custom-binary-ccs || die "Failed to build kernel package."
-cd .. || die "Can't chdir to ../ ."
+cd ../ || die "Can't chdir to ../ ."
 
 # Install header package for compiling additional modules.
 dpkg -i linux-headers-${VERSION}-ccs_${VERSION}.*_i386.deb || die "Can't install packages."
-cd linux-ubuntu-modules-2.6.22-2.6.22 || die "Can't chdir to linux-ubuntu-modules-2.6.22-2.6.22 ."
+cd linux-ubuntu-modules-2.6.24-2.6.24 || die "Can't chdir to linux-ubuntu-modules-2.6.24-2.6.24 ."
 awk ' BEGIN { flag = 0; print ""; } { if ($1 == "Package:" ) { if (index($0, "-generic") > 0) flag = 1; else flag = 0; }; if (flag) print $0; } ' debian/control.stub | sed -e 's:-generic:-ccs:' > debian/control.stub.ccs || die "Can't create file."
 cat debian/control.stub.ccs >> debian/control.stub || die "Can't edit file."
 debian/rules debian/control || die "Can't run control."
@@ -64,7 +70,7 @@ sed -i -e 's:virtual:virtual ccs:' debian/rules.d/i386.mk || die "Can't edit fil
 debian/rules binary-indep binary-arch || die "Failed to build kernel package."
 cd .. || die "Can't chdir to ../ ."
 
-cd linux-restricted-modules-2.6.22-2.6.22.4/ || die "Can't chdir to linux-restricted-modules-2.6.22-2.6.22.4/ ."
+cd linux-restricted-modules-2.6.24-2.6.24.14/ || die "Can't chdir to linux-restricted-modules-2.6.24-2.6.24.14/ ."
 awk ' BEGIN { flag = 0; print ""; } { if ( $1 == "Package:") { if ( index($2, "-generic") > 0) flag = 1; else flag = 0; }; if (flag) print $0; } ' debian/control.stub.in | sed -e 's:-generic:-ccs:g' > debian/control.stub.in.tmp || die "Can't create file."
 cat debian/control.stub.in.tmp >> debian/control.stub.in || die "Can't edit file."
 sed -i -e 's/,generic/,ccs generic/' debian/rules || die "Can't edit file."
