@@ -3,15 +3,19 @@
  *
  * Common functions for SAKURA and TOMOYO.
  *
- * Copyright (C) 2005-2007  NTT DATA CORPORATION
+ * Copyright (C) 2005-2008  NTT DATA CORPORATION
  *
- * Version: 1.5.1   2007/10/19
+ * Version: 1.5.5   2008/09/03
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
  *
  */
 
+#include <linux/version.h>
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
+#define __KERNEL_SYSCALLS__
+#endif
 #include <linux/string.h>
 #include <linux/mm.h>
 #include <linux/utime.h>
@@ -20,8 +24,7 @@
 #include <linux/slab.h>
 #include <asm/uaccess.h>
 #include <stdarg.h>
-#include <linux/version.h>
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 0)
 #include <linux/namei.h>
 #include <linux/mount.h>
 static const int lookup_flags = LOOKUP_FOLLOW;
@@ -32,6 +35,12 @@ static const int lookup_flags = LOOKUP_FOLLOW | LOOKUP_POSITIVE;
 #include <linux/ccs_common.h>
 #include <linux/ccs_proc.h>
 #include <linux/tomoyo.h>
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
+#include <linux/unistd.h>
+#endif
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24)
+#define find_task_by_pid find_task_by_vpid
+#endif
 
 #ifdef CONFIG_TOMOYO_MAX_ACCEPT_ENTRY
 #define MAX_ACCEPT_ENTRY (CONFIG_TOMOYO_MAX_ACCEPT_ENTRY)
@@ -474,16 +483,19 @@ int io_printf(struct io_buffer *head, const char *fmt, ...)
  */
 const char *GetEXE(void)
 {
-	if (current->mm) {
-		struct vm_area_struct *vma = current->mm->mmap;
-		while (vma) {
-			if ((vma->vm_flags & VM_EXECUTABLE) && vma->vm_file) {
-				return realpath_from_dentry(vma->vm_file->f_dentry, vma->vm_file->f_vfsmnt);
-			}
-			vma = vma->vm_next;
+	struct mm_struct *mm = current->mm;
+	struct vm_area_struct *vma;
+	const char *cp = NULL;
+	if (!mm) return NULL;
+	down_read(&mm->mmap_sem);
+	for (vma = mm->mmap; vma; vma = vma->vm_next) {
+		if ((vma->vm_flags & VM_EXECUTABLE) && vma->vm_file) {
+			cp = realpath_from_dentry(vma->vm_file->f_dentry, vma->vm_file->f_vfsmnt);
+			break;
 		}
 	}
-	return NULL;
+	up_read(&mm->mmap_sem);
+	return cp;
 }
 
 const char *GetMSG(const int is_enforce)
@@ -503,7 +515,6 @@ unsigned int CheckCCSFlags(const unsigned int index)
 #endif
 		&& profile_ptr[profile] ? profile_ptr[profile]->value[index] : 0;
 }
-EXPORT_SYMBOL(CheckCCSFlags);
 
 unsigned int TomoyoVerboseMode(void)
 {
@@ -515,7 +526,6 @@ unsigned int CheckCCSEnforce(const unsigned int index)
 {
 	return CheckCCSFlags(index) == 3;
 }
-EXPORT_SYMBOL(CheckCCSEnforce);
 
 unsigned int CheckDomainQuota(struct domain_info * const domain)
 {
@@ -539,7 +549,6 @@ unsigned int CheckCCSAccept(const unsigned int index, struct domain_info * const
 	if (CheckCCSFlags(index) != 1) return 0;
 	return CheckDomainQuota(domain);
 }
-EXPORT_SYMBOL(CheckCCSAccept);
 
 static struct profile *FindOrAssignNewProfile(const unsigned int profile)
 {
@@ -550,7 +559,7 @@ static struct profile *FindOrAssignNewProfile(const unsigned int profile)
 		if ((ptr = alloc_element(sizeof(*ptr))) != NULL) {
 			int i;
 			for (i = 0; i < CCS_MAX_CONTROL_INDEX; i++) ptr->value[i] = ccs_control_array[i].current_value;
-			mb(); /* Instead of using spinlock. */
+			mb(); /* Avoid out-of-order execution. */
 			profile_ptr[profile] = ptr;
 		}
 	}
@@ -690,7 +699,7 @@ static int AddManagerEntry(const char *manager, u8 is_delete)
 	if ((new_entry = alloc_element(sizeof(*new_entry))) == NULL) goto out;
 	new_entry->manager = saved_manager;
 	new_entry->is_domain = is_domain;
-	mb(); /* Instead of using spinlock. */
+	mb(); /* Avoid out-of-order execution. */
 	if ((ptr = policy_manager_list) != NULL) {
 		while (ptr->next) ptr = ptr->next; ptr->next = new_entry;
 	} else {
@@ -906,7 +915,7 @@ static int ReadDomainPolicy(struct io_buffer *head)
 						if (acltype2paths(acl_type) == 2) {
 							struct double_acl_record *ptr2 = (struct double_acl_record *) ptr;
 							const u8 b0 = ptr2->u1_is_group, b1 = ptr2->u2_is_group;
-							if (io_printf(head, "allow_%s %s%s %s%s", keyword, 
+							if (io_printf(head, "allow_%s %s%s %s%s", keyword,
 								      b0 ? "@" : "", b0 ? ptr2->u1.group1->group_name->name : ptr2->u1.filename1->name,
 								      b1 ? "@" : "", b1 ? ptr2->u2.group2->group_name->name : ptr2->u2.filename2->name)
 							    || DumpCondition(head, ptr->cond)) {
@@ -1141,8 +1150,6 @@ static int ReadSystemPolicy(struct io_buffer *head)
 
 /*************************  POLICY LOADER  *************************/
 
-static int profile_loaded = 0;
-
 static const char *ccs_loader = NULL;
 
 static int __init CCS_loader_Setup(char *str)
@@ -1153,6 +1160,22 @@ static int __init CCS_loader_Setup(char *str)
 
 __setup("CCS_loader=", CCS_loader_Setup);
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
+static int run_ccs_loader(void *unused)
+{
+	char *argv[2];
+	char *envp[3];
+	printk(KERN_INFO "Calling %s to load policy. Please wait.\n",
+	       ccs_loader);
+	argv[0] = (char *) ccs_loader;
+	argv[1] = NULL;
+	envp[0] = "HOME=/";
+	envp[1] = "PATH=/sbin:/bin:/usr/sbin:/usr/bin";
+	envp[2] = NULL;
+	return exec_usermodehelper(argv[0], argv, envp);
+}
+#endif
+
 void CCS_LoadPolicy(const char *filename)
 {
 	if (sbin_init_started) return;
@@ -1176,9 +1199,14 @@ void CCS_LoadPolicy(const char *filename)
 			printk("Not activating Mandatory Access Control now since %s doesn't exist.\n", ccs_loader);
 			return;
 		}
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 25)
+		path_put(&nd.path);
+#else
 		path_release(&nd);
+#endif
 	}
-	if (!profile_loaded) {
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 0)
+	{
 		char *argv[2], *envp[3];
 		printk("Calling %s to load policy. Please wait.\n", ccs_loader);
 		argv[0] = (char *) ccs_loader;
@@ -1186,23 +1214,53 @@ void CCS_LoadPolicy(const char *filename)
 		envp[0] = "HOME=/";
 		envp[1] = "PATH=/sbin:/bin:/usr/sbin:/usr/bin";
 		envp[2] = NULL;
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
 		call_usermodehelper(argv[0], argv, envp, 1);
-#else
-		call_usermodehelper(argv[0], argv, envp);
-#endif
-		while (!profile_loaded) {
-			set_current_state(TASK_INTERRUPTIBLE);
-			schedule_timeout(HZ / 10);
-		}
 	}
+#elif defined(TASK_DEAD)
+	{
+		/* Copied from kernel/kmod.c */
+		struct task_struct *task = current;
+		pid_t pid = kernel_thread(run_ccs_loader, NULL, 0);
+		sigset_t tmpsig;
+		spin_lock_irq(&task->sighand->siglock);
+		tmpsig = task->blocked;
+		siginitsetinv(&task->blocked,
+			      sigmask(SIGKILL) | sigmask(SIGSTOP));
+		recalc_sigpending();
+		spin_unlock_irq(&current->sighand->siglock);
+		if (pid >= 0)
+			waitpid(pid, NULL, __WCLONE);
+		spin_lock_irq(&task->sighand->siglock);
+		task->blocked = tmpsig;
+		recalc_sigpending();
+		spin_unlock_irq(&task->sighand->siglock);
+	}
+#else
+	{
+		/* Copied from kernel/kmod.c */
+		struct task_struct *task = current;
+		pid_t pid = kernel_thread(run_ccs_loader, NULL, 0);
+		sigset_t tmpsig;
+		spin_lock_irq(&task->sigmask_lock);
+		tmpsig = task->blocked;
+		siginitsetinv(&task->blocked,
+			      sigmask(SIGKILL) | sigmask(SIGSTOP));
+		recalc_sigpending(task);
+		spin_unlock_irq(&task->sigmask_lock);
+		if (pid >= 0)
+			waitpid(pid, NULL, __WCLONE);
+		spin_lock_irq(&task->sigmask_lock);
+		task->blocked = tmpsig;
+		recalc_sigpending(task);
+		spin_unlock_irq(&task->sigmask_lock);
+	}
+#endif
 #ifdef CONFIG_SAKURA
-	printk("SAKURA: 1.5.1   2007/10/19\n");
+	printk("SAKURA: 1.5.5   2008/09/03\n");
 #endif
 #ifdef CONFIG_TOMOYO
-	printk("TOMOYO: 1.5.1   2007/10/19\n");
+	printk("TOMOYO: 1.5.5   2008/09/03\n");
 #endif
-	//if (!profile_loaded) panic("No profiles loaded. Run policy loader using 'init=' option.\n");
 	printk("Mandatory Access Control activated.\n");
 	sbin_init_started = 1;
 	ccs_log_level = KERN_WARNING;
@@ -1210,7 +1268,7 @@ void CCS_LoadPolicy(const char *filename)
 		struct domain_info *domain;
 		for (domain = &KERNEL_DOMAIN; domain; domain = domain->next) {
 			const u8 profile = domain->profile;
-			if (!profile_ptr[profile]) panic("Profile %u (used by '%s') not defined.\n", profile, domain->domainname->name); 
+			if (!profile_ptr[profile]) panic("Profile %u (used by '%s') not defined.\n", profile, domain->domainname->name);
 		}
 	}
 }
@@ -1442,7 +1500,7 @@ static int ReadUpdatesCounter(struct io_buffer *head)
 static int ReadVersion(struct io_buffer *head)
 {
 	if (!head->read_eof) {
-		if (io_printf(head, "1.5.1") == 0) head->read_eof = 1;
+		if (io_printf(head, "1.5.5") == 0) head->read_eof = 1;
 	}
 	return 0;
 }
@@ -1532,14 +1590,31 @@ int CCS_OpenControl(const int type, struct file *file)
 		head->read = ReadUpdatesCounter;
 		break;
 	}
-	if (type != CCS_GRANTLOG && type != CCS_REJECTLOG && type != CCS_QUERY) {
+	if (!(file->f_mode & FMODE_READ)) {
+		/*
+		 * No need to allocate read_buf since it is not opened
+		 * for reading.
+		 */
+		head->read = NULL;
+		head->poll = NULL;
+	} else if (type != CCS_GRANTLOG && type != CCS_REJECTLOG && type != CCS_QUERY) {
+		/*
+		 * Don't allocate buffer for reading if the file is one of
+		 * /proc/ccs/grant_log , /proc/ccs/reject_log , /proc/ccs/query.
+		 */
 		if (!head->readbuf_size) head->readbuf_size = PAGE_SIZE * 2;
 		if ((head->read_buf = ccs_alloc(head->readbuf_size)) == NULL) {
 			ccs_free(head);
 			return -ENOMEM;
 		}
 	}
-	if (head->write) {
+	if (!(file->f_mode & FMODE_WRITE)) {
+		/*
+		 * No need to allocate write_buf since it is not opened
+		 * for writing.
+		 */
+		head->write = NULL;
+	} else if (head->write) {
 		head->writebuf_size = PAGE_SIZE * 2;
 		if ((head->write_buf = ccs_alloc(head->writebuf_size)) == NULL) {
 			ccs_free(head->read_buf);
@@ -1549,7 +1624,7 @@ int CCS_OpenControl(const int type, struct file *file)
 	}
 	file->private_data = head;
 	if (type == CCS_SELFDOMAIN) CCS_ReadControl(file, NULL, 0);
-	else if (head->write == WriteAnswer) atomic_inc(&queryd_watcher);
+	else if (head->write == WriteAnswer || head->read == ReadQuery) atomic_inc(&queryd_watcher);
 	return 0;
 }
 
@@ -1624,8 +1699,7 @@ int CCS_WriteControl(struct file *file, const char __user *buffer, const int buf
 int CCS_CloseControl(struct file *file)
 {
 	struct io_buffer *head = file->private_data;
-	if (head->write == WriteAnswer) atomic_dec(&queryd_watcher);
-	else if (head->read == ReadMemoryCounter) profile_loaded = 1;
+	if (head->write == WriteAnswer || head->read == ReadQuery) atomic_dec(&queryd_watcher);
 	ccs_free(head->read_buf); head->read_buf = NULL;
 	ccs_free(head->write_buf); head->write_buf = NULL;
 	ccs_free(head); head = NULL;
