@@ -5,7 +5,7 @@
  *
  * Copyright (C) 2005-2008  NTT DATA CORPORATION
  *
- * Version: 1.5.3   2008/01/31
+ * Version: 1.5.5   2008/09/03
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -16,7 +16,7 @@
 #include <linux/ccs_common.h>
 #include <linux/sakura.h>
 #include <linux/realpath.h>
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 0)
 #include <linux/namei.h>
 #else
 #include <linux/fs.h>
@@ -71,12 +71,21 @@ static int AddChrootACL(const char *dir, const int is_delete)
 	return error;
 }
 
-int CheckChRootPermission(struct nameidata *nd)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 27)
+#define PATH_or_NAMEIDATA path
+#else
+#define PATH_or_NAMEIDATA nameidata
+#endif
+int CheckChRootPermission(struct PATH_or_NAMEIDATA *path)
 {
 	int error = -EPERM;
 	char *root_name;
 	if (!CheckCCSFlags(CCS_SAKURA_RESTRICT_CHROOT)) return 0;
-	root_name = realpath_from_dentry(nd->dentry, nd->mnt);
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 25) && LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 26)
+	root_name = realpath_from_dentry(path->path.dentry, path->path.mnt);
+#else
+	root_name = realpath_from_dentry(path->dentry, path->mnt);
+#endif
 	if (root_name) {
 		struct path_info dir;
 		dir.name = root_name;
