@@ -3,9 +3,9 @@
  *
  * Implementation of the Domain-Based Mandatory Access Control.
  *
- * Copyright (C) 2005-2007  NTT DATA CORPORATION
+ * Copyright (C) 2005-2008  NTT DATA CORPORATION
  *
- * Version: 1.4   2007/04/01
+ * Version: 1.5.5   2008/09/03
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -40,44 +40,43 @@ DECLARE_MUTEX(domain_acl_lock);
 
 /***** The structure for program files to force domain reconstruction. *****/
 
-typedef struct domain_initializer_entry {
+struct domain_initializer_entry {
 	struct domain_initializer_entry *next;
 	const struct path_info *domainname;    /* This may be NULL */
 	const struct path_info *program;
 	u8 is_deleted;
 	u8 is_not;
 	u8 is_last_name;
-	u8 is_oldstyle;
-} DOMAIN_INITIALIZER_ENTRY;
+};
 
 /***** The structure for domains to not to transit domains. *****/
 
-typedef struct domain_keeper_entry {
+struct domain_keeper_entry {
 	struct domain_keeper_entry *next;
 	const struct path_info *domainname;
 	const struct path_info *program;       /* This may be NULL */
 	u8 is_deleted;
 	u8 is_not;
 	u8 is_last_name;
-} DOMAIN_KEEPER_ENTRY;
+};
 
 /***** The structure for program files that should be aggregated. *****/
 
-typedef struct aggregator_entry {
+struct aggregator_entry {
 	struct aggregator_entry *next;
 	const struct path_info *original_name;
 	const struct path_info *aggregated_name;
 	int is_deleted;
-} AGGREGATOR_ENTRY;
+};
 
 /***** The structure for program files that should be aliased. *****/
 
-typedef struct alias_entry {
+struct alias_entry {
 	struct alias_entry *next;
 	const struct path_info *original_name;
 	const struct path_info *aliased_name;
 	int is_deleted;
-} ALIAS_ENTRY;
+};
 
 /*************************  VARIABLES  *************************/
 
@@ -86,12 +85,6 @@ static DECLARE_MUTEX(new_domain_assign_lock);
 
 /*************************  UTILITY FUNCTIONS  *************************/
 
-int IsDomainDef(const unsigned char *buffer)
-{
-	/* while (*buffer && (*buffer <= ' ' || *buffer >= 127)) buffer++; */
-	return strncmp(buffer, ROOT_NAME, ROOT_NAME_LEN) == 0;
-}
-
 const char *GetLastName(const struct domain_info *domain)
 {
 	const char *cp0 = domain->domainname->name, *cp1;
@@ -99,18 +92,9 @@ const char *GetLastName(const struct domain_info *domain)
 	return cp0;
 }
 
-int ReadSelfDomain(IO_BUFFER *head)
-{
-	if (!head->read_eof) {
-		io_printf(head, "%s", current->domain_info->domainname->name);
-		head->read_eof = 1;
-	}
-	return 0;
-}
-
 int AddDomainACL(struct acl_info *ptr, struct domain_info *domain, struct acl_info *new_ptr)
 {
-	mb(); /* Instead of using spinlock. */
+	mb(); /* Avoid out-of-order execution. */
 	if (!ptr) domain->first_acl_ptr = (struct acl_info *) new_ptr;
 	else ptr->next = (struct acl_info *) new_ptr;
 	UpdateCounter(CCS_UPDATES_COUNTER_DOMAIN_POLICY);
@@ -124,29 +108,13 @@ int DelDomainACL(struct acl_info *ptr)
 	return 0;
 }
 
-int TooManyDomainACL(struct domain_info * const domain) {
-	unsigned int count = 0;
-	struct acl_info *ptr;
-	for (ptr = domain->first_acl_ptr; ptr; ptr = ptr->next) {
-		if (!ptr->is_deleted) count++;
-	}
-	/* If there are so many entries, don't append if accept mode. */
-	if (count < CheckCCSFlags(CCS_TOMOYO_MAX_ACCEPT_ENTRY)) return 0;
-	if (!domain->quota_warned) {
-		printk("TOMOYO-WARNING: Domain '%s' has so many ACLs to hold. Stopped auto-append mode.\n", domain->domainname->name);
-		domain->quota_warned = 1;
-	}
-	return 1;
-}
-
-
 /*************************  DOMAIN INITIALIZER HANDLER  *************************/
 
-static DOMAIN_INITIALIZER_ENTRY *domain_initializer_list = NULL;
+static struct domain_initializer_entry *domain_initializer_list = NULL;
 
-static int AddDomainInitializerEntry(const char *domainname, const char *program, const int is_not, const int is_delete, const int is_oldstyle)
+static int AddDomainInitializerEntry(const char *domainname, const char *program, const int is_not, const int is_delete)
 {
-	DOMAIN_INITIALIZER_ENTRY *new_entry, *ptr;
+	struct domain_initializer_entry *new_entry, *ptr;
 	static DECLARE_MUTEX(lock);
 	const struct path_info *saved_program, *saved_domainname = NULL;
 	int error = -ENOMEM;
@@ -163,7 +131,7 @@ static int AddDomainInitializerEntry(const char *domainname, const char *program
 	if ((saved_program = SaveName(program)) == NULL) return -ENOMEM;
 	down(&lock);
 	for (ptr = domain_initializer_list; ptr; ptr = ptr->next) {
-		if (ptr->is_not == is_not && ptr->is_oldstyle == is_oldstyle && ptr->domainname == saved_domainname && ptr->program == saved_program) {
+		if (ptr->is_not == is_not && ptr->domainname == saved_domainname && ptr->program == saved_program) {
 			ptr->is_deleted = is_delete;
 			error = 0;
 			goto out;
@@ -173,13 +141,12 @@ static int AddDomainInitializerEntry(const char *domainname, const char *program
 		error = -ENOENT;
 		goto out;
 	}
-	if ((new_entry = (DOMAIN_INITIALIZER_ENTRY *) alloc_element(sizeof(DOMAIN_INITIALIZER_ENTRY))) == NULL) goto out;
+	if ((new_entry = alloc_element(sizeof(*new_entry))) == NULL) goto out;
 	new_entry->domainname = saved_domainname;
 	new_entry->program = saved_program;
 	new_entry->is_not = is_not;
 	new_entry->is_last_name = is_last_name;
-	new_entry->is_oldstyle = is_oldstyle;
-	mb(); /* Instead of using spinlock. */
+	mb(); /* Avoid out-of-order execution. */
 	if ((ptr = domain_initializer_list) != NULL) {
 		while (ptr->next) ptr = ptr->next; ptr->next = new_entry;
 	} else {
@@ -191,17 +158,17 @@ static int AddDomainInitializerEntry(const char *domainname, const char *program
 	return error;
 }
 
-int ReadDomainInitializerPolicy(IO_BUFFER *head)
+int ReadDomainInitializerPolicy(struct io_buffer *head)
 {
-	DOMAIN_INITIALIZER_ENTRY *ptr = (DOMAIN_INITIALIZER_ENTRY *) head->read_var2;
+	struct domain_initializer_entry *ptr = head->read_var2;
 	if (!ptr) ptr = domain_initializer_list;
 	while (ptr) {
-		head->read_var2 = (void *) ptr;
+		head->read_var2 = ptr;
 		if (!ptr->is_deleted) {
 			if (ptr->domainname) {
-				if (io_printf(head, "%s%s%s from %s\n", ptr->is_not ? "no_" : "", ptr->is_oldstyle ? KEYWORD_INITIALIZER : KEYWORD_INITIALIZE_DOMAIN, ptr->program->name, ptr->domainname->name)) break;
+				if (io_printf(head, "%s" KEYWORD_INITIALIZE_DOMAIN "%s from %s\n", ptr->is_not ? "no_" : "", ptr->program->name, ptr->domainname->name)) break;
 			} else {
-				if (io_printf(head, "%s%s%s\n", ptr->is_not ? "no_" : "", ptr->is_oldstyle ? KEYWORD_INITIALIZER : KEYWORD_INITIALIZE_DOMAIN, ptr->program->name)) break;
+				if (io_printf(head, "%s" KEYWORD_INITIALIZE_DOMAIN "%s\n", ptr->is_not ? "no_" : "", ptr->program->name)) break;
 			}
 		}
 		ptr = ptr->next;
@@ -209,20 +176,20 @@ int ReadDomainInitializerPolicy(IO_BUFFER *head)
 	return ptr ? -ENOMEM : 0;
 }
 
-int AddDomainInitializerPolicy(char *data, const int is_not, const int is_delete, const int is_oldstyle)
+int AddDomainInitializerPolicy(char *data, const int is_not, const int is_delete)
 {
 	char *cp = strstr(data, " from ");
 	if (cp) {
 		*cp = '\0';
-		return AddDomainInitializerEntry(cp + 6, data, is_not, is_delete, is_oldstyle);
+		return AddDomainInitializerEntry(cp + 6, data, is_not, is_delete);
 	} else {
-		return AddDomainInitializerEntry(NULL, data, is_not, is_delete, is_oldstyle);
+		return AddDomainInitializerEntry(NULL, data, is_not, is_delete);
 	}
 }
 
 static int IsDomainInitializer(const struct path_info *domainname, const struct path_info *program, const struct path_info *last_name)
 {
-	DOMAIN_INITIALIZER_ENTRY *ptr;
+	struct domain_initializer_entry *ptr;
 	int flag = 0;
 	for (ptr = domain_initializer_list; ptr; ptr = ptr->next) {
 		if (ptr->is_deleted ) continue;
@@ -242,11 +209,11 @@ static int IsDomainInitializer(const struct path_info *domainname, const struct
 
 /*************************  DOMAIN KEEPER HANDLER  *************************/
 
-static DOMAIN_KEEPER_ENTRY *domain_keeper_list = NULL;
+static struct domain_keeper_entry *domain_keeper_list = NULL;
 
 static int AddDomainKeeperEntry(const char *domainname, const char *program, const int is_not, const int is_delete)
 {
-	DOMAIN_KEEPER_ENTRY *new_entry, *ptr;
+	struct domain_keeper_entry *new_entry, *ptr;
 	const struct path_info *saved_domainname, *saved_program = NULL;
 	static DECLARE_MUTEX(lock);
 	int error = -ENOMEM;
@@ -273,12 +240,12 @@ static int AddDomainKeeperEntry(const char *domainname, const char *program, con
 		error = -ENOENT;
 		goto out;
 	}
-	if ((new_entry = (DOMAIN_KEEPER_ENTRY *) alloc_element(sizeof(DOMAIN_KEEPER_ENTRY))) == NULL) goto out;
+	if ((new_entry = alloc_element(sizeof(*new_entry))) == NULL) goto out;
 	new_entry->domainname = saved_domainname;
 	new_entry->program = saved_program;
 	new_entry->is_not = is_not;
 	new_entry->is_last_name = is_last_name;
-	mb(); /* Instead of using spinlock. */
+	mb(); /* Avoid out-of-order execution. */
 	if ((ptr = domain_keeper_list) != NULL) {
 		while (ptr->next) ptr = ptr->next; ptr->next = new_entry;
 	} else {
@@ -301,12 +268,12 @@ int AddDomainKeeperPolicy(char *data, const int is_not, const int is_delete)
 	}
 }
 
-int ReadDomainKeeperPolicy(IO_BUFFER *head)
+int ReadDomainKeeperPolicy(struct io_buffer *head)
 {
-	DOMAIN_KEEPER_ENTRY *ptr = (DOMAIN_KEEPER_ENTRY *) head->read_var2;
+	struct domain_keeper_entry *ptr = head->read_var2;
 	if (!ptr) ptr = domain_keeper_list;
 	while (ptr) {
-		head->read_var2 = (void *) ptr;
+		head->read_var2 = ptr;
 		if (!ptr->is_deleted) {
 			if (ptr->program) {
 				if (io_printf(head, "%s" KEYWORD_KEEP_DOMAIN "%s from %s\n", ptr->is_not ? "no_" : "", ptr->program->name, ptr->domainname->name)) break;
@@ -321,14 +288,14 @@ int ReadDomainKeeperPolicy(IO_BUFFER *head)
 
 static int IsDomainKeeper(const struct path_info *domainname, const struct path_info *program, const struct path_info *last_name)
 {
-	DOMAIN_KEEPER_ENTRY *ptr;
+	struct domain_keeper_entry *ptr;
 	int flag = 0;
 	for (ptr = domain_keeper_list; ptr; ptr = ptr->next) {
 		if (ptr->is_deleted) continue;
 		if (!ptr->is_last_name) {
 			if (ptr->domainname != domainname) continue;
 		} else {
-			if (pathcmp(ptr->domainname, last_name)) continue; 
+			if (pathcmp(ptr->domainname, last_name)) continue;
 		}
 		if (ptr->program && pathcmp(ptr->program, program)) continue;
 		if (ptr->is_not) return 0;
@@ -339,11 +306,11 @@ static int IsDomainKeeper(const struct path_info *domainname, const struct path_
 
 /*************************  SYMBOLIC LINKED PROGRAM HANDLER  *************************/
 
-static ALIAS_ENTRY *alias_list = NULL;
+static struct alias_entry *alias_list = NULL;
 
 static int AddAliasEntry(const char *original_name, const char *aliased_name, const int is_delete)
 {
-	ALIAS_ENTRY *new_entry, *ptr;
+	struct alias_entry *new_entry, *ptr;
 	static DECLARE_MUTEX(lock);
 	const struct path_info *saved_original_name, *saved_aliased_name;
 	int error = -ENOMEM;
@@ -361,10 +328,10 @@ static int AddAliasEntry(const char *original_name, const char *aliased_name, co
 		error = -ENOENT;
 		goto out;
 	}
-	if ((new_entry = (ALIAS_ENTRY *) alloc_element(sizeof(ALIAS_ENTRY))) == NULL) goto out;
+	if ((new_entry = alloc_element(sizeof(*new_entry))) == NULL) goto out;
 	new_entry->original_name = saved_original_name;
 	new_entry->aliased_name = saved_aliased_name;
-	mb(); /* Instead of using spinlock. */
+	mb(); /* Avoid out-of-order execution. */
 	if ((ptr = alias_list) != NULL) {
 		while (ptr->next) ptr = ptr->next; ptr->next = new_entry;
 	} else {
@@ -376,12 +343,12 @@ static int AddAliasEntry(const char *original_name, const char *aliased_name, co
 	return error;
 }
 
-int ReadAliasPolicy(IO_BUFFER *head)
+int ReadAliasPolicy(struct io_buffer *head)
 {
-	ALIAS_ENTRY *ptr = (ALIAS_ENTRY *) head->read_var2;
+	struct alias_entry *ptr = head->read_var2;
 	if (!ptr) ptr = alias_list;
 	while (ptr) {
-		head->read_var2 = (void *) ptr;
+		head->read_var2 = ptr;
 		if (!ptr->is_deleted && io_printf(head, KEYWORD_ALIAS "%s %s\n", ptr->original_name->name, ptr->aliased_name->name)) break;
 		ptr = ptr->next;
 	}
@@ -398,11 +365,11 @@ int AddAliasPolicy(char *data, const int is_delete)
 
 /*************************  DOMAIN AGGREGATOR HANDLER  *************************/
 
-static AGGREGATOR_ENTRY *aggregator_list = NULL;
+static struct aggregator_entry *aggregator_list = NULL;
 
 static int AddAggregatorEntry(const char *original_name, const char *aggregated_name, const int is_delete)
 {
-	AGGREGATOR_ENTRY *new_entry, *ptr;
+	struct aggregator_entry *new_entry, *ptr;
 	static DECLARE_MUTEX(lock);
 	const struct path_info *saved_original_name, *saved_aggregated_name;
 	int error = -ENOMEM;
@@ -420,10 +387,10 @@ static int AddAggregatorEntry(const char *original_name, const char *aggregated_
 		error = -ENOENT;
 		goto out;
 	}
-	if ((new_entry = (AGGREGATOR_ENTRY *) alloc_element(sizeof(AGGREGATOR_ENTRY))) == NULL) goto out;
+	if ((new_entry = alloc_element(sizeof(*new_entry))) == NULL) goto out;
 	new_entry->original_name = saved_original_name;
 	new_entry->aggregated_name = saved_aggregated_name;
-	mb(); /* Instead of using spinlock. */
+	mb(); /* Avoid out-of-order execution. */
 	if ((ptr = aggregator_list) != NULL) {
 		while (ptr->next) ptr = ptr->next; ptr->next = new_entry;
 	} else {
@@ -435,12 +402,12 @@ static int AddAggregatorEntry(const char *original_name, const char *aggregated_
 	return error;
 }
 
-int ReadAggregatorPolicy(IO_BUFFER *head)
+int ReadAggregatorPolicy(struct io_buffer *head)
 {
-	AGGREGATOR_ENTRY *ptr = (AGGREGATOR_ENTRY *) head->read_var2;
+	struct aggregator_entry *ptr = head->read_var2;
 	if (!ptr) ptr = aggregator_list;
 	while (ptr) {
-		head->read_var2 = (void *) ptr;
+		head->read_var2 = ptr;
 		if (!ptr->is_deleted && io_printf(head, KEYWORD_AGGREGATOR "%s %s\n", ptr->original_name->name, ptr->aggregated_name->name)) break;
 		ptr = ptr->next;
 	}
@@ -457,7 +424,7 @@ int AddAggregatorPolicy(char *data, const int is_delete)
 
 /*************************  DOMAIN DELETION HANDLER  *************************/
 
-// #define DEBUG_DOMAIN_UNDELETE
+/* #define DEBUG_DOMAIN_UNDELETE */
 
 int DeleteDomain(char *domainname0)
 {
@@ -537,23 +504,6 @@ struct domain_info *UndeleteDomain(const char *domainname0)
 
 /*************************  DOMAIN TRANSITION HANDLER  *************************/
 
-struct domain_info *FindDomain(const char *domainname0)
-{
-	struct domain_info *domain;
-	static int first = 1;
-	struct path_info domainname;
-	domainname.name = domainname0;
-	fill_path_info(&domainname);
-	if (first) {
-		KERNEL_DOMAIN.domainname = SaveName(ROOT_NAME);
-		first = 0;
-	}
-	for (domain = &KERNEL_DOMAIN; domain; domain = domain->next) {
-		if (!domain->is_deleted && !pathcmp(&domainname, domain->domainname)) return domain;
-	}
-	return NULL;
-}
-
 struct domain_info *FindOrAssignNewDomain(const char *domainname, const u8 profile)
 {
 	struct domain_info *domain = NULL;
@@ -583,16 +533,16 @@ struct domain_info *FindOrAssignNewDomain(const char *domainname, const u8 profi
 		for (ptr = domain->first_acl_ptr; ptr; ptr = ptr->next) ptr->is_deleted = 1;
 		domain->profile = profile;
 		domain->quota_warned = 0;
-		mb(); /* Instead of using spinlock. */
+		mb(); /* Avoid out-of-order execution. */
 		domain->is_deleted = 0;
 		goto out;
 	}
 	/* No memory reusable. Create using new memory. */
-	if ((domain = (struct domain_info *) alloc_element(sizeof(struct domain_info))) != NULL) {
+	if ((domain = alloc_element(sizeof(*domain))) != NULL) {
 		struct domain_info *ptr = &KERNEL_DOMAIN;
 		domain->domainname = saved_domainname;
 		domain->profile = profile;
-		mb(); /* Instead of using spinlock. */
+		mb(); /* Avoid out-of-order execution. */
 		while (ptr->next) ptr = ptr->next; ptr->next = domain;
 	}
  out: ;
@@ -631,34 +581,48 @@ static int Escape(char *dest, const char *src, int dest_len)
 
 static char *get_argv0(struct linux_binprm *bprm)
 {
-	if (bprm->argc > 0) {
-		char *arg_ptr = ccs_alloc(PAGE_SIZE);
-		int arg_len = 0;
-		const unsigned long pos = bprm->p;
-		int i = pos / PAGE_SIZE, offset = pos % PAGE_SIZE;
-		if (!arg_ptr) goto out;
-		while (1) {
-			struct page *page = bprm->page[i];
+	char *arg_ptr = ccs_alloc(PAGE_SIZE); /* Initial buffer. */
+	int arg_len = 0;
+	unsigned long pos = bprm->p;
+	int i = pos / PAGE_SIZE, offset = pos % PAGE_SIZE;
+	if (!bprm->argc || !arg_ptr) goto out;
+	while (1) {
+		struct page *page;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 23) && defined(CONFIG_MMU)
+		if (get_user_pages(current, bprm->mm, pos, 1, 0, 1, &page, NULL) <= 0) goto out;
+#else
+		page = bprm->page[i];
+#endif
+		{ /* Map and copy to kernel buffer and unmap. */
 			const char *kaddr = kmap(page);
-			if (!kaddr) goto out;
+			if (!kaddr) { /* Mapping failed. */
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 23) && defined(CONFIG_MMU)
+				put_page(page);
+#endif
+				goto out;
+			}
 			memmove(arg_ptr + arg_len, kaddr + offset, PAGE_SIZE - offset);
 			kunmap(page);
-			arg_len += PAGE_SIZE - offset;
-			if (memchr(arg_ptr, '\0', arg_len)) break;
-			{
-				char *tmp_arg_ptr = ccs_alloc(arg_len + PAGE_SIZE);
-				if (!tmp_arg_ptr) goto out;
-				memmove(tmp_arg_ptr, arg_ptr, arg_len);
-				ccs_free(arg_ptr);
-				arg_ptr = tmp_arg_ptr;
-			}
-			i++;
-			offset = 0;
 		}
-		return arg_ptr;
-	out:
-		ccs_free(arg_ptr);
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 23) && defined(CONFIG_MMU)
+		put_page(page);
+		pos += PAGE_SIZE - offset;
+#endif
+		arg_len += PAGE_SIZE - offset;
+		if (memchr(arg_ptr, '\0', arg_len)) break;
+		{ /* Initial buffer was too small for argv[0]. Retry after expanding buffer. */
+			char *tmp_arg_ptr = ccs_alloc(arg_len + PAGE_SIZE);
+			if (!tmp_arg_ptr) goto out;
+			memmove(tmp_arg_ptr, arg_ptr, arg_len);
+			ccs_free(arg_ptr);
+			arg_ptr = tmp_arg_ptr;
+		}
+		i++;
+		offset = 0;
 	}
+	return arg_ptr;
+ out: /* Release initial buffer. */
+	ccs_free(arg_ptr);
 	return NULL;
 }
 
@@ -681,8 +645,8 @@ static int FindNextDomain(struct linux_binprm *bprm, struct domain_info **next_d
 		 */
 		static int first = 1;
 		if (first) {
-			AddDomainInitializerEntry(NULL, "/sbin/hotplug", 0, 0, 0);
-			AddDomainInitializerEntry(NULL, "/sbin/modprobe", 0, 0, 0);
+			AddDomainInitializerEntry(NULL, "/sbin/hotplug", 0, 0);
+			AddDomainInitializerEntry(NULL, "/sbin/modprobe", 0, 0);
 			first = 0;
 		}
 	}
@@ -703,7 +667,7 @@ static int FindNextDomain(struct linux_binprm *bprm, struct domain_info **next_d
 
 	/* Check 'alias' directive. */
 	if (pathcmp(&r, &s)) {
-		ALIAS_ENTRY *ptr;
+		struct alias_entry *ptr;
 		/* Is this program allowed to be called via symbolic links? */
 		for (ptr = alias_list; ptr; ptr = ptr->next) {
 			if (ptr->is_deleted || pathcmp(&r, ptr->original_name) || pathcmp(&s, ptr->aliased_name)) continue;
@@ -713,10 +677,30 @@ static int FindNextDomain(struct linux_binprm *bprm, struct domain_info **next_d
 			break;
 		}
 	}
-	
+
+	/* Compare basename of real_program_name and argv[0] */
+	if (bprm->argc > 0 && CheckCCSFlags(CCS_TOMOYO_MAC_FOR_ARGV0)) {
+		char *org_argv0 = get_argv0(bprm);
+		retval = -ENOMEM;
+		if (org_argv0) {
+			const int len = strlen(org_argv0);
+			char *printable_argv0 = ccs_alloc(len * 4 + 8);
+			if (printable_argv0 && Escape(printable_argv0, org_argv0, len * 4 + 8) == 0) {
+				const char *base_argv0, *base_filename;
+				if ((base_argv0 = strrchr(printable_argv0, '/')) == NULL) base_argv0 = printable_argv0; else base_argv0++;
+				if ((base_filename = strrchr(real_program_name, '/')) == NULL) base_filename = real_program_name; else base_filename++;
+				if (strcmp(base_argv0, base_filename)) retval = CheckArgv0Perm(&r, base_argv0);
+				else retval = 0;
+			}
+			ccs_free(printable_argv0);
+			ccs_free(org_argv0);
+		}
+		if (retval) goto out;
+	}
+
 	/* Check 'aggregator' directive. */
 	{
-		AGGREGATOR_ENTRY *ptr;
+		struct aggregator_entry *ptr;
 		/* Is this program allowed to be aggregated? */
 		for (ptr = aggregator_list; ptr; ptr = ptr->next) {
 			if (ptr->is_deleted || !PathMatchesToPattern(&r, ptr->original_name)) continue;
@@ -727,33 +711,13 @@ static int FindNextDomain(struct linux_binprm *bprm, struct domain_info **next_d
 		}
 	}
 
-	/* Compare basename of symlink_program_name and argv[0] */
-	if (bprm->argc > 0 && CheckCCSFlags(CCS_TOMOYO_MAC_FOR_ARGV0)) {
-		char *org_argv0 = get_argv0(bprm);
-		retval = -ENOMEM;
-		if (org_argv0) {
-			const int len = strlen(org_argv0);
-			char *printable_argv0 = ccs_alloc(len * 4 + 8);
-			if (printable_argv0 && Escape(printable_argv0, org_argv0, len * 4 + 8) == 0) {
-				const char *base_argv0, *base_filename;
-				if ((base_argv0 = strrchr(printable_argv0, '/')) == NULL) base_argv0 = printable_argv0; else base_argv0++;
-				if ((base_filename = strrchr(symlink_program_name, '/')) == NULL) base_filename = symlink_program_name; else base_filename++;
-				if (strcmp(base_argv0, base_filename)) retval = CheckArgv0Perm(&s, base_argv0);
-				else retval = 0;
-			}
-			ccs_free(printable_argv0);
-			ccs_free(org_argv0);
-		}
-		if (retval) goto out;
-	}
-	
 	/* Check execute permission. */
 	if ((retval = CheckExecPerm(&r, filp)) < 0) goto out;
 
-	/* Allocate memory for calcurating domain name. */
+	/* Allocate memory for calculating domain name. */
 	retval = -ENOMEM;
 	if ((new_domain_name = ccs_alloc(CCS_MAX_PATHNAME_LEN + 16)) == NULL) goto out;
-	
+
 	if (IsDomainInitializer(old_domain->domainname, &r, &l)) {
 		/* Transit to the child of KERNEL_DOMAIN domain. */
 		snprintf(new_domain_name, CCS_MAX_PATHNAME_LEN + 1, ROOT_NAME " " "%s", real_program_name);
@@ -797,7 +761,7 @@ static int FindNextDomain(struct linux_binprm *bprm, struct domain_info **next_d
 int search_binary_handler_with_transition(struct linux_binprm *bprm, struct pt_regs *regs)
 {
 	struct domain_info *next_domain = NULL, *prev_domain = current->domain_info;
- 	int retval;
+	int retval;
 #if defined(CONFIG_SAKURA) || defined(CONFIG_TOMOYO)
 	extern void CCS_LoadPolicy(const char *filename);
 	CCS_LoadPolicy(bprm->filename);
